require 'spec_helper'

describe Search do
  before do
    @search = Search.new(start: nil, generator: nil, scheduler: nil)
  end

  it "is an object" do
    @search.should be
  end

  describe "run" do
    it "executes the callback with start node on first iteration" do
      generator = Proc.new do |state|
        [state+1]
      end
      scheduler = Proc.new{|states| states.first}
      generator.should_receive(:call).with(0).and_return([1])
      @search = Search.new(start: 0, generator: generator, scheduler: scheduler)
      @search.iterations.should == 0
      @search.run
      @search.iterations.should == 1
    end
    it "maintains a current state" do
      generator = Proc.new do |state|
        [state+1]
      end
      scheduler = Proc.new{|states| states.first}
      @search = Search.new(start: 0, generator: generator, scheduler: scheduler)
      @search.current_state.should == 0
      @search.run
      @search.current_state.should == 1
    end
    it "maintains a parent tree state" do
      generator = Proc.new do |state|
        [state+1]
      end
      scheduler = Proc.new{|states| states.first}
      @search = Search.new(start: 0, generator: generator, scheduler: scheduler)
      @search.run
      @search.path.should == [0,1]
      @search.distance.should == 1
      @search.run
      @search.path.should == [0,1,2]
      @search.distance.should == 2
    end
  end
end
